///////////////////////////////////////////////////////////////////////////////
// FileParameterRecord.cpp
// Paul Wilson
// Oxford Metrics, May 2000
///////////////////////////////////////////////////////////////////////////////

#include <limits>
#include <cassert>

#include "FileParameterRecord.h"

VFileParameterRecord::VFileParameterRecord(BYTE * a_RawData, int a_Length) : VFileRecord(a_RawData, a_Length)
{
	m_strName = "";
	m_Type = EUndefined;	
	m_Dimensions.reserve(3);
}

VFileParameterRecord::~VFileParameterRecord()
{
	// class specific clean-up... then call the base class destructor
	//VFileRecord::~VFileRecord();
}
	
void VFileParameterRecord::SetName(std::string a_strName)
{
	assert(a_strName != "");

	m_strName = a_strName;
	SetEdited();
}

void VFileParameterRecord::SetType(EDataType a_Type)
{
	m_Type = a_Type;
	SetEdited();
}

void VFileParameterRecord::DeleteDimension(int a_Index)
{
	assert(a_Index >= 0 && a_Index < m_Dimensions.size());

	std::vector<SSHORT>::iterator DimIt = m_Dimensions.begin();

	for (int i = 0; i < a_Index; i++)
		DimIt++;

	m_Dimensions.erase(DimIt);
	SetEdited();
}

void VFileParameterRecord::AddDimension(SSHORT a_NewDimension)
{
	assert(a_NewDimension > 0);

	m_Dimensions.push_back(a_NewDimension);
	SetEdited();
}

void VFileParameterRecord::InsertDimension(SSHORT a_NewDimension, int a_Index)
{
	assert(a_NewDimension > 0);
	assert(a_Index >= 0 && a_Index < m_Dimensions.size());

	std::vector<SSHORT>::iterator DimIt = m_Dimensions.begin();

	for (int i = 0; i < a_Index; i++)
		DimIt++;

	m_Dimensions.insert(DimIt, a_NewDimension);
	SetEdited();
}

void VFileParameterRecord::SetDimensionAt(SSHORT a_NewDimension, int a_Index)
{
	assert(a_NewDimension > 0);
	assert(a_Index >= 0 && a_Index < m_Dimensions.size());

	m_Dimensions[a_Index] = a_NewDimension;
	SetEdited();
}

int VFileParameterRecord::CalculateLength() const
{
	int Length = sizeof(BYTE) +			// Name len
				 m_strName.length() + 	// name 
				 1 +					// (+ null terminator)
				 sizeof(BYTE) +			// Type
				 sizeof(BYTE);			// Num dimensions

	// And for the variable half of the record...

	Length += m_Dimensions.size() * sizeof(SSHORT);

	// the number of data entries multiplied by the width of the datatype

	Length += GetNumDataEntries() * GetDataTypeWidth();

	return Length;
}

int VFileParameterRecord::GetNumDataEntries() const
{
	// CHECK THIS!!

	std::vector<SSHORT>::const_iterator DimIt;
	int NumEntries = 1;

//	for (DimIt = const_cast<std::vector<SSHORT>::iterator>(m_Dimensions.begin()); DimIt != m_Dimensions.end(); DimIt++)
	for (DimIt = m_Dimensions.begin(); DimIt != m_Dimensions.end(); DimIt++)
		NumEntries *= *DimIt;

	return NumEntries;
}

void VFileParameterRecord::BuildRecordFromBytes()
{
	// When doing this... remember to store the position of the start of the data
	
	int ByteIndex = 0;

	BYTE NameLen = m_RawData[ByteIndex];
	ByteIndex += sizeof(BYTE);

	m_strName = (char *)&(m_RawData[ByteIndex]);	// Is this dodgy?
	ByteIndex += NameLen;

	m_Type = (EDataType)m_RawData[ByteIndex];
	ByteIndex += sizeof(BYTE);

	BYTE NumDims = m_RawData[ByteIndex];
	ByteIndex += sizeof(BYTE);

	SSHORT * pDims = (SSHORT *)&m_RawData[ByteIndex];
	ByteIndex += NumDims * sizeof(SSHORT);
	m_Dimensions.insert(m_Dimensions.begin(), pDims, pDims + NumDims);

	int NumEntries = GetNumDataEntries();

	if (m_pDataValues != NULL)
	{
		delete [] m_pDataValues;
	}

	m_pDataValues = new BYTE[NumEntries];
	memcpy(m_pDataValues, &m_RawData[ByteIndex], NumEntries);
}

SSHORT VFileParameterRecord::BuildBytesFromRecord()
{
	SSHORT OldSize = GetLength();
	SSHORT NewSize = CalculateLength();

	if (OldSize != NewSize)
	{
		assert(false);
		delete [] m_RawData;	// This doesn't work... we'll be deleting all the data

		m_RawData = new BYTE[NewSize];
	}

	assert(m_RawData);

	if (m_RawData)
	{
		// Okay, we've got the storage necessary, proceed..

		int ByteIndex = 0;

		assert(m_strName.length() + 1 < std::numeric_limits<BYTE>::max());	// Check this assert...
		m_RawData[ByteIndex] = (BYTE)m_strName.length() + 1;
		ByteIndex += sizeof(BYTE);

		memcpy(&m_RawData[ByteIndex], (BYTE *)m_strName.c_str(), m_strName.length() + 1);
		ByteIndex += m_strName.length() + 1;

		assert(m_Type < std::numeric_limits<BYTE>::max());	// Check this assert...
		m_RawData[ByteIndex] = (BYTE)m_Type;
		ByteIndex += sizeof(BYTE);

		m_RawData[ByteIndex] = m_Dimensions.size();
		ByteIndex += sizeof(BYTE);

		std::vector<SSHORT>::iterator DimIt;

		for (DimIt = m_Dimensions.begin(); DimIt != m_Dimensions.end(); DimIt++)
		{
			memcpy(&m_RawData[ByteIndex], (BYTE *)&(*DimIt), sizeof(SSHORT));
			ByteIndex += sizeof(SSHORT);
		}

		int NumEntries = GetNumDataEntries();

		for (int i = 0; i < NumEntries; i++)
		{
			m_RawData[ByteIndex++] = m_pDataValues[i];
		}

		assert(ByteIndex == NewSize);

		SetLength(NewSize);
	}
	else
	{
		// We've run out of memory... pants
		SetLength(0);
		NewSize = 0;
	}

	return NewSize;
}

int VFileParameterRecord::GetDataTypeWidth() const
{
	int DataWidth = 0;

	switch(m_Type)
	{
	case EBinary:
	case EText:
	case EBoolean:
		DataWidth = sizeof(BYTE);
		break;

	case EStructure:
	case EShort:
		DataWidth = sizeof(SSHORT);
		break;

	case ELong:
		DataWidth = sizeof(SLONG);
		break;

	case EFloat:
		DataWidth = sizeof(float);
		break;

	case EDouble:
		DataWidth = sizeof(double);
		break;

	default:
		assert(false);
	}	

	return DataWidth;
}

bool VFileParameterRecord::GetBoolData(int a_Index) const
{
	assert(a_Index >= 0);

	return m_pDataValues[a_Index] != 0;
}

BYTE VFileParameterRecord::GetBinaryData(int a_Index) const
{
	assert(a_Index >= 0);

	return m_pDataValues[a_Index];
}

SSHORT VFileParameterRecord::GetShortData(int a_Index) const
{
	assert(a_Index >= 0);

	SSHORT * pIndex = (SSHORT *)m_pDataValues;
	return pIndex[a_Index];
}

SLONG VFileParameterRecord::GetLongData(int a_Index) const
{
	assert(a_Index >= 0);

	SLONG * pIndex = (SLONG *)m_pDataValues;
	return pIndex[a_Index];
}

float VFileParameterRecord::GetFloatData(int a_Index) const
{
	assert(a_Index >= 0);

	float * pIndex = (float *)m_pDataValues;
	return pIndex[a_Index];
}

double VFileParameterRecord::GetDoubleData(int a_Index) const
{
	assert(a_Index >= 0);

	double * pIndex = (double *)m_pDataValues;
	return pIndex[a_Index];
}

std::string VFileParameterRecord::GetTextData(int a_Index) const
{
	int Size = 1;

	if (m_Dimensions.size() > 0)
		Size = m_Dimensions[0];

	assert(a_Index >= 0 && a_Index < (GetNumDataEntries() / Size));

	// Adjust the index to take the length dimension into account

	char * pIndex = (char *)&m_pDataValues[a_Index * Size];	// Check this... (unsigned to signed??)

	std::string strVal = pIndex;			
		
	if (strVal.length() + 1 > Size)
		strVal.resize(Size - 1);

	return strVal;
}

void VFileParameterRecord::SetBoolData(int a_Index, bool a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	m_pDataValues[a_Index] = (a_Value != 0);

	SetEdited();
}

void VFileParameterRecord::SetBinaryData(int a_Index, BYTE a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	m_pDataValues[a_Index] = a_Value;

	SetEdited();
}

void VFileParameterRecord::SetShortData(int a_Index, SSHORT a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	SSHORT * pShort = (SSHORT *)m_pDataValues;
	pShort[a_Index] = a_Value;

	SetEdited();
}

void VFileParameterRecord::SetLongData(int a_Index, SLONG a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	SLONG * pLong = (SLONG *)m_pDataValues;
	pLong[a_Index] = a_Value;

	SetEdited();
}

void VFileParameterRecord::SetFloatData(int a_Index, float a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	float * pFloat = (float *)m_pDataValues;
	pFloat[a_Index] = a_Value;

	SetEdited();
}

void VFileParameterRecord::SetDoubleData(int a_Index, double a_Value)
{
	assert(a_Index >= 0 && a_Index < GetNumDataEntries());

	double * pDouble = (double *)m_pDataValues;
	pDouble[a_Index] = a_Value;

	SetEdited();
}

void VFileParameterRecord::SetTextData(int a_Index, std::string a_Value)
{
	int Size = 1;

	if (m_Dimensions.size() > 0)
		Size = m_Dimensions[0];

	assert(a_Index >= 0 && a_Index < (GetNumDataEntries() / Size));

	if (a_Value.size() != Size)
		a_Value.resize(Size, '\0');

	// Adjust the index to take the length dimension into account

	BYTE * pIndex = &m_pDataValues[a_Index * Size];

	memcpy(pIndex, a_Value.c_str(), a_Value.length() + 1);

	SetEdited();
}

