//////////////////////////////////////////////////////////////////////////////////
//	VFileTranslator.cpp
//	Oxford Metrics
//	July 2000
//////////////////////////////////////////////////////////////////////////////////


#include "VFileTranslator.h"
#include "CreateOffsetCubes.h"
#include "CopyKeys.h"

#include <maya/MStatus.h>
#include <maya/MTime.h>
#include <maya/MAnimControl.h>
#include <maya/MFnDagNode.h>
#include <maya/MPlugArray.h>
#include <maya/MGlobal.h>
#include <maya/MDistance.h>
#include <maya/MDagPath.h>
#include <maya/MSelectionList.h>
#include "SpaceLocator.h"
#include "Body.h"
#include "ToLocal.h"

//////////////////////////////////////////////////////////////////////////////////
MStatus initializePlugin(MObject obj)
{
	MStatus status;
	MFnPlugin plugin(obj, "Oxford Metrics Ltd.", "Release 1.0", "Any", &status);

	if(MS::kSuccess != status)
	{
		status.perror("createFileTranslator");
		return status;
	}

	// Register the translator with the system
	status = plugin.registerFileTranslator(VFILE_PLUGIN_NAME, "", VFileTranslator::Creator, NULL, "", true);

	if (MS::kSuccess != status) 
		status.perror("registerFileTranslator");

	if (plugin.registerCommand(CREATE_OFFSET_CUBES, VCreateOffsetCubes::creator) != MS::kSuccess)
	{
		status.perror("registerCommand");
	}

	if (plugin.registerCommand(COPY_KEYS, VCopyKeys::creator) != MS::kSuccess)
	{
		status.perror("registerCommand");
	}

	if (plugin.registerCommand(TO_LOCAL, VToLocal::creator) != MS::kSuccess)
	{
		status.perror("registerCommand");
	}

	plugin.registerNode( DOF_LOCATOR, VDOFLocator::id, &VDOFLocator::creator, 
								  &VDOFLocator::initialize, MPxNode::kLocatorNode );

	plugin.registerNode( MARKER_LOCATOR, VMarkerLocator::id, &VMarkerLocator::creator, 
								  &VMarkerLocator::initialize, MPxNode::kLocatorNode );
	
	return status;
}

MStatus uninitializePlugin(MObject obj)
{
	MStatus   status;
	MFnPlugin plugin(obj);

	status = plugin.deregisterFileTranslator(VFILE_PLUGIN_NAME);

	if (!status) 
	{
		status.perror("deregisterFileTranslator");
	}

	if (plugin.deregisterCommand(CREATE_OFFSET_CUBES) != MS::kSuccess)
	{
		status.perror("deregisterCommand");
	}

	if (plugin.deregisterCommand(COPY_KEYS) != MS::kSuccess)
	{
		status.perror("deregisterCommand");
	}

	if (plugin.deregisterCommand(TO_LOCAL) != MS::kSuccess)
	{
		status.perror("deregisterCommand");
	}
	
	plugin.deregisterNode(VDOFLocator::id);
	plugin.deregisterNode(VMarkerLocator::id);

	return status;
}
//////////////////////////////////////////////////////////////////////////////////


VFileTranslator::VFileTranslator()
{
	m_bRegisterNodeFail = false;
}

VFileTranslator::~VFileTranslator()
{
	CleanUp();
}

void VFileTranslator::CleanUp()
{
	std::vector<VMayaBody *>::iterator iBody;

	for (iBody = m_Bodies.begin(); iBody != m_Bodies.end(); iBody++)
	{
		VMayaBody * pBody = *iBody;
		delete pBody;
	}
}

void * VFileTranslator::Creator()
{
	VFileTranslator * pNewTranslator = new VFileTranslator();

	return pNewTranslator;
}

MString VFileTranslator::defaultExtension() const
{
	return "V";
}

MString VFileTranslator::filter() const
{
	return "*.V";
}

MPxFileTranslator::MFileKind VFileTranslator::identifyFile(const MFileObject & a_rFile, const char * a_pContents, short a_SizeOfBuf) const 
{
	std::string strFilename = a_rFile.name().toUpperCase().asChar();

	int DotPos = strFilename.rfind(".");

	if (strFilename.substr(DotPos + 1) == "V")
		return kIsMyFileType;
	else
		return kNotMyFileType;
}

MStatus VFileTranslator::reader(const MFileObject & file, const MString & optionsString, MPxFileTranslator::FileAccessMode mode) 
{
	// In case this isn't the first time this has been run... delete old objects

	m_Bodies.clear();

	m_FileManager.OpenVFile(file.resolvedFullName().asChar());
	VFileContents * pContents = m_FileManager.ReadVFile();
	m_FileManager.CloseVFile();

	bool bZUp = MGlobal::isZAxisUp();
	VMayaBody::EOrient eUpAxis = bZUp ? VMayaBody::EZ_Up : VMayaBody::EY_Up;

	// Set the min and max times in Maya (on the anim timebar). May have to adjust 
	// this to take motion scaling into account
	
	float FrameRate = 1.0;
	int NumDatagroups = pContents->GetNumDatagroups();

	// This is rather crude - it's possible that this will not work correctly
	// for complex datasets. But, the worst thing that will happen is that the replay
	// speed will have to be adjusted...

	if (NumDatagroups)
		FrameRate = pContents->GetFrameRate(0);

	int MaxFrame = pContents->GetMaxFrameNum();
	double Scalar = GetMayaFrameRate() / FrameRate;
	MTime MinTime;
	MTime MaxTime(((double)MaxFrame) * Scalar, MTime::uiUnit());

	if (MAnimControl::maxTime() < MaxTime)
		MAnimControl::setMinMaxTime(MinTime, MaxTime);

	// Okay.. deal with each datagroup seperately

	std::vector<std::string> DofLabels;
	std::string newname;

	for (int GroupIdx = 0; GroupIdx < NumDatagroups; GroupIdx++)
	{
		std::vector<std::string> GlobalBodies;
		std::vector<std::string> LocalBodies;
		std::vector<std::string> Markers;

		if (pContents->GetLabelsForDatagroup(GroupIdx, DofLabels))
		{
			// Extract the body (or marker) labels from the dof labels

			ExtractBodyAndMarkerLabels(DofLabels, GlobalBodies, LocalBodies, Markers);

			// Now step through each body label and create a locator

			std::vector<std::string>::iterator LabelIt;
			std::vector<std::string>::iterator GlobalBodyIt;
			std::vector<std::string>::iterator LocalBodyIt;

			MDagModifier dagMod;
			MSelectionList SelList;

			for (GlobalBodyIt = GlobalBodies.begin(); GlobalBodyIt != GlobalBodies.end(); GlobalBodyIt++)
			{
				std::string strGlobalName = *GlobalBodyIt;// + "_global"; //_global makes V files incomatible with RT stream.
				std::string strBodyName, strGroupName;
				
				DeriveGroupAndLocatorName(strGlobalName, strGroupName, strBodyName);
		
				if (strGroupName != "")
					strGlobalName = "|" + strGroupName + "|" + strBodyName;
				else
					strGlobalName = strBodyName;
		
				// Attempt to select the locator...

				MObject locObj = FindOrCreateDofLocator(strGlobalName, strGroupName, strBodyName);

				VMayaBody * pNewBody = new VMayaBody(strGlobalName, GroupIdx, eUpAxis);

				// We've created an object to hold the dofs for one body... now scan
				// the labels and see which dof's pertain to this body. (could be in ANY order)

				int DofIndex = 0;

				for (LabelIt = DofLabels.begin(); LabelIt != DofLabels.end(); LabelIt++, DofIndex++)
				{
					if (LabelIt->find(*GlobalBodyIt) == 0 && IsGlobal(*LabelIt))
					{
						// now figure out which dof this is...

						std::string strDof = LabelIt->substr(GlobalBodyIt->length());
						VMayaBody::EDof DofType = VMayaBody::EUndefDof;

						if ((DofType = pNewBody->ConvertDofSuffix(strDof, strDof)) != VMayaBody::EUndefDof)
						{
							pNewBody->SetDofIndex(DofType, DofIndex);

							MFnDagNode LocNode(locObj);
							MPlug Plug = LocNode.findPlug(strDof.c_str());

							MObject AnimCurveObject = GetAnimCurve(Plug);
							MFnAnimCurve * pAnimCurve = new MFnAnimCurve(AnimCurveObject);
						
							pNewBody->SetAnimCurve(DofType, pAnimCurve);
						}
					}
				}

				m_Bodies.push_back(pNewBody);
			}

			// Same for local bodies...

			for (LocalBodyIt = LocalBodies.begin(); LocalBodyIt != LocalBodies.end(); LocalBodyIt++)
			{
				std::string strGlobalName = *LocalBodyIt + "_local";
				std::string strBodyName, strGroupName;
				
				DeriveGroupAndLocatorName(strGlobalName, strGroupName, strBodyName);
		
				if (strGroupName != "")
					strGlobalName = "|" + strGroupName + "|" + strBodyName;
				else
					strGlobalName = strBodyName;
		
				// Attempt to select the locator...

				MObject locObj = FindOrCreateDofLocator(strGlobalName, strGroupName, strBodyName);

				VMayaBody * pNewBody = new VMayaBody(strGlobalName, GroupIdx, eUpAxis);

				// We've created an object to hold the dofs for one body... now scan
				// the labels and see which dof's pertain to this body. (could be in ANY order)

				int DofIndex = 0;

				for (LabelIt = DofLabels.begin(); LabelIt != DofLabels.end(); LabelIt++, DofIndex++)
				{
					if (LabelIt->find(*LocalBodyIt) == 0 && !IsGlobal(*LabelIt))
					{
						// now figure out which dof this is...

						std::string strDof = LabelIt->substr(LocalBodyIt->length());
						VMayaBody::EDof DofType = VMayaBody::EUndefDof;

						if ((DofType = pNewBody->ConvertDofSuffix(strDof, strDof)) != VMayaBody::EUndefDof)
						{
							pNewBody->SetDofIndex(DofType, DofIndex);

							MStatus Status1;
							MFnDagNode LocNode(locObj);
							MPlug Plug = LocNode.findPlug(strDof.c_str(), &Status1);

							assert(Status1 == MStatus::kSuccess);

							MObject AnimCurveObject = GetAnimCurve(Plug);
							MFnAnimCurve * pAnimCurve = new MFnAnimCurve(AnimCurveObject);
						
							pNewBody->SetAnimCurve(DofType, pAnimCurve);
						}
					}
				}

				m_Bodies.push_back(pNewBody);
			}

			std::vector<std::string>::iterator MarkerIt;

			for (MarkerIt = Markers.begin(); MarkerIt != Markers.end(); MarkerIt++)
			{
				std::string strGlobalName = *MarkerIt + "_marker";
				std::string strMarkerName, strGroupName;
				
				DeriveGroupAndLocatorName(strGlobalName, strGroupName, strMarkerName);
		
				if (strGroupName != "")
					strGlobalName = "|" + strGroupName + "|" + strMarkerName;
				else
					strGlobalName = strMarkerName;
		
				// Attempt to select the locator...

				if (strGroupName == "")
					strGroupName = "UnnamedCharacter";

				MObject locObj = CreateMarkerLocator(strGlobalName, strGroupName, strMarkerName);

				VMayaBody * pNewBody = new VMayaBody(strGlobalName, GroupIdx, eUpAxis);

				// We've created an object to hold the dofs for one marker... now scan
				// the labels and see which dof's pertain to this marker. (could be in ANY order)

				int DofIndex = 0;

				for (LabelIt = DofLabels.begin(); LabelIt != DofLabels.end(); LabelIt++, DofIndex++)
				{
					std::string strMarker = *MarkerIt;

					if (LabelIt->find(strMarker) == 0)
					{
						// now figure out which dof this is...

						std::string strDof = LabelIt->substr(strMarker.length());
						VMayaBody::EDof DofType = VMayaBody::EUndefDof;

						if ((DofType = pNewBody->ConvertDofSuffix(strDof, strDof)) != VMayaBody::EUndefDof)
						{
							pNewBody->SetDofIndex(DofType, DofIndex);

							MFnDagNode LocNode(locObj);
							MPlug Plug = LocNode.findPlug(strDof.c_str());

							MObject AnimCurveObject = GetAnimCurve(Plug);
							MFnAnimCurve * pAnimCurve = new MFnAnimCurve(AnimCurveObject);
						
							pNewBody->SetAnimCurve(DofType, pAnimCurve);
						}
					}
				}

				m_Bodies.push_back(pNewBody);
			}
		
		}

		SetupKeyframes(pContents, GroupIdx);	
	}

	CleanUp();
	delete pContents;

	return MS::kSuccess;
}

void VFileTranslator::DeriveGroupAndLocatorName(std::string i_strGlobalName, std::string & o_strGroupName, std::string & o_strBodyName)
{
	int ColonPos = i_strGlobalName.find(":");

	if (ColonPos != -1)
	{
		o_strBodyName = i_strGlobalName.substr(ColonPos + 1);
		o_strGroupName = i_strGlobalName.substr(0, ColonPos);

		int NumericStart = o_strGroupName.find_first_of("1234567890");

		if (NumericStart != -1)
			o_strGroupName = o_strGroupName.substr(0, NumericStart);
	}
	else
	{
		o_strBodyName = i_strGlobalName;
		o_strGroupName = "";
	}
}

MObject VFileTranslator::FindOrCreateDofLocator(std::string i_strGlobalName, std::string i_strGroupName, std::string i_strBodyName)
{
	MObject locObj, Parent;
	MDagModifier dagMod;
	MSelectionList SelList;

	MGlobal::setActiveSelectionList(SelList);

	// Search for a node which is either |GroupName|BodyName... 
	// ...or |GroupName|Parent|Parent...|BodyName

	// First... see if we can find the group node

	if (MGlobal::selectByName(i_strGroupName.c_str()) != MStatus::kSuccess)
	{
		// No group node... create one... and then create a locator

		std::string GroupCmd = "group -em -n " + i_strGroupName;
		MGlobal::executeCommand(GroupCmd.c_str());

		if (MGlobal::selectByName(i_strGroupName.c_str()) != MStatus::kSuccess)
			assert(false);

		// Get hold of parent

		MGlobal::getActiveSelectionList(SelList);
		assert(SelList.length() == 1);
		SelList.getDependNode(0, Parent);
		
		// now create locator

		locObj = dagMod.createNode("DOFLocator");

		dagMod.renameNode(locObj, i_strBodyName.c_str());
		dagMod.reparentNode(locObj, Parent);
		dagMod.doIt();
	}
	else
	{
		// Okay... found a group node... recurse through the nodes below to find our locator

		MGlobal::getActiveSelectionList(SelList);

		locObj = FindLocatorRecursive(i_strBodyName, i_strGroupName);

		if (locObj.isNull())
		{
			// Get hold of parent

			int length = SelList.length();
			assert(SelList.length() == 1);
			SelList.getDependNode(0, Parent);

			// create a new node

			locObj = dagMod.createNode("DOFLocator");

			dagMod.renameNode(locObj, i_strBodyName.c_str());
			dagMod.reparentNode(locObj, Parent);
			dagMod.doIt();
		}
	}

	return locObj;
}

MObject VFileTranslator::FindLocatorRecursive(std::string i_strLocatorName, std::string i_strParentName)
{
	MObject locObj;
	MSelectionList SelList;

	MGlobal::setActiveSelectionList(SelList);

	if (i_strParentName[0] != '|')
		i_strParentName = "|" + i_strParentName;

	std::string strGetChildren = "listRelatives -c -f ";	
	std::string strChild = "";

	MStringArray ChildStrings;
	MGlobal::executeCommand((strGetChildren + i_strParentName).c_str(), ChildStrings);

	int NumChildren = ChildStrings.length();

	for (int i = 0; i < NumChildren; i++)
	{
		strChild = ChildStrings[i].asChar();

		if (IsLocatorName(strChild))
		{
			int BarPos = strChild.rfind("|");

			assert(BarPos != -1);

			if (BarPos != -1)
			{
				std::string strCheckName = strChild.substr(BarPos + 1);

				if (strCheckName == i_strLocatorName)
				{
					// hurray... we've found it!... return it

					MGlobal::setActiveSelectionList(SelList);

					if (MGlobal::selectByName(strChild.c_str()) == MStatus::kSuccess)
					{
						MGlobal::getActiveSelectionList(SelList);
						assert(SelList.length() == 1);
						SelList.getDependNode(0, locObj);						

						// Also... before we return.. delete all the keys on this node

						std::string DeleteKeyCmd = "cutKey -cl -t \":\" -hierarchy none {\"" + strChild + "\"}";
						MGlobal::executeCommand(DeleteKeyCmd.c_str());

						return locObj;
					}
					else
						assert(!"Could not select the child locator... very strange");
				}
			}

			locObj = FindLocatorRecursive(i_strLocatorName, strChild);

			if (!locObj.isNull())
				return locObj;
		}
	}
	
	return MObject::kNullObj;
}

bool VFileTranslator::IsLocatorName(std::string a_strObjectName)
{
	bool bRetVal = false;

	std::string strGetChildren = "listRelatives -c -f ";	

	MStringArray ChildStrings;
	MGlobal::executeCommand((strGetChildren + a_strObjectName).c_str(), ChildStrings);

	int NumChildren = ChildStrings.length();

	for (int i = 0; i < NumChildren; i++)
	{
		std::string strChild = ChildStrings[i].asChar();

		int BarPos = strChild.rfind("|");

		assert(BarPos != -1);

		if (BarPos != -1)
		{
			strChild = strChild.substr(BarPos + 1);

			if (strChild.find(DOF_LOCATOR) != -1 || strChild.find(MARKER_LOCATOR) != -1)
			{
				bRetVal = true;
				break;
			}
		}
	}

	return bRetVal;
}

MObject VFileTranslator::CreateMarkerLocator(std::string i_strGlobalName, std::string i_strGroupName, std::string i_strMarkerName)
{
	MObject locObj, Parent;
	MDagModifier dagMod;
	MSelectionList SelList;

	MGlobal::setActiveSelectionList(SelList);

	// No locator... must create one... see if there is a group node...

	if (MGlobal::selectByName(i_strGroupName.c_str()) != MStatus::kSuccess)
	{
		// No group node... create one...

//		if (i_strGroupName != "")
//			i_strGroupName = "V_Import";

		std::string GroupCmd = "group -em -n " + i_strGroupName;
		MGlobal::executeCommand(GroupCmd.c_str());

		if (MGlobal::selectByName(i_strGroupName.c_str()) != MStatus::kSuccess)
			assert(false);
	}

	MGlobal::getActiveSelectionList(SelList);
#ifdef _DEBUG
	int length = SelList.length();
#endif
	assert(SelList.length() == 1);
	SelList.getDependNode(0, Parent);						

	locObj = dagMod.createNode("MarkerLocator");
	dagMod.renameNode(locObj, i_strMarkerName.c_str());
	dagMod.reparentNode(locObj, Parent);
	dagMod.doIt();

	return locObj;
}

void VFileTranslator::CreateOffsetCubes(std::vector<std::string> & a_rLocators)
{
/*	MGlobal::executeCommand("currentTime 50");
	MGlobal::executeCommand("select -cl");

	std::vector<std::string>::iterator iLocator;

	for (iLocator = a_rLocators.begin(); iLocator != a_rLocators.end(); iLocator++)
	{
		
	}
*/
}

MObject VFileTranslator::GetAnimCurve(MPlug & a_rCurrentPlug)
{
	MPlug NonConstPlug(a_rCurrentPlug);
	MObject animCurveObj;

	//	If nothing is connected, make a new Anim Curve.
	//
	if (!NonConstPlug.isConnected()) 
	{
		MFnAnimCurve animCurve;
		animCurveObj = animCurve.create(NonConstPlug);
	} 
	else 
	{
		MStatus Status1;
		MPlugArray ConnArray;
		NonConstPlug.connectedTo(ConnArray, true, false, &Status1);

		assert(Status1 == MStatus::kSuccess);

		bool bFound = false;
		unsigned NumConnections = ConnArray.length();

		if (NumConnections > 0)
		{
			for (unsigned i = 0; !bFound && i < NumConnections; i++) 
			{
				MObject srcNodeObj = ConnArray[i].node();

				//	If the connected node is an anim curve, then use that node.

				switch(srcNodeObj.apiType()) 
				{
				case MFn::kAnimCurve:
				case MFn::kAnimCurveTimeToAngular:
				case MFn::kAnimCurveTimeToDistance:
				case MFn::kAnimCurveTimeToTime:
				case MFn::kAnimCurveTimeToUnitless:
				case MFn::kAnimCurveUnitlessToAngular:
				case MFn::kAnimCurveUnitlessToDistance:
				case MFn::kAnimCurveUnitlessToTime:
				case MFn::kAnimCurveUnitlessToUnitless:
					animCurveObj = srcNodeObj;
					bFound = true;
					break;
				default:
					break;
				}
			}
		}
		else
		{
			MFnAnimCurve animCurve;
			animCurveObj = animCurve.create(NonConstPlug);
		}
	}

	return animCurveObj;
}

//void VFileTranslator::ExtractBodyAndMarkerLabels(std::vector<std::string> & a_rDofLabels, std::vector<std::string> & a_rBodyLabels, std::vector<std::string> & a_rMarkerLabels)
void VFileTranslator::ExtractBodyAndMarkerLabels(std::vector<std::string> & i_rDofLabels, std::vector<std::string> & o_rGlobalBodyLabels, std::vector<std::string> & o_rLocalBodyLabels, std::vector<std::string> & o_rMarkerLabels)
{
	bool bFound;
	std::vector<std::string>::iterator LabelIt;
	std::vector<std::string>::iterator BodyIt;
	std::vector<std::string>::iterator MarkerIt;

	for (LabelIt = i_rDofLabels.begin(); LabelIt != i_rDofLabels.end(); LabelIt++)
	{
		bFound = false;
		std::string ObjectName = StripSuffix(*LabelIt);

		if (IsBody(*LabelIt))
		{
			// It's a body.. 
			
			if (IsGlobal(*LabelIt))
			{
				// if unique, add to the body vector

				for (BodyIt = o_rGlobalBodyLabels.begin(); !bFound && BodyIt != o_rGlobalBodyLabels.end(); BodyIt++)
				{
					if (_stricmp(ObjectName.c_str(), BodyIt->c_str()) == 0) 
					{
						bFound = true; 
						break;
					}
				}

				if (!bFound)
				{
					// This body name does not yet exist in our body vector... so add it!

					o_rGlobalBodyLabels.push_back(ObjectName);
				}
			}
			else
			{
				// It's Local!
				// if unique, add to the body vector

				for (BodyIt = o_rLocalBodyLabels.begin(); !bFound && BodyIt != o_rLocalBodyLabels.end(); BodyIt++)
				{
					if (_stricmp(ObjectName.c_str(), BodyIt->c_str()) == 0) 
					{
						bFound = true; 
						break;
					}
				}

				if (!bFound)
				{
					// This body name does not yet exist in our body vector... so add it!

					o_rLocalBodyLabels.push_back(ObjectName);
				}
			}
		}
		else
		{
			// It's a marker... if unique, add to the marker vector

			for (MarkerIt = o_rMarkerLabels.begin(); !bFound && MarkerIt != o_rMarkerLabels.end(); MarkerIt++)
			{
				if (_stricmp(ObjectName.c_str(), MarkerIt->c_str()) == 0) 
				{
					bFound = true; 
					break;
				}
			}

			if (!bFound)
			{
				// This body name does not yet exist in our body vector... so add it!

				o_rMarkerLabels.push_back(ObjectName);
			}
		}
	}
}

bool VFileTranslator::IsBody(std::string a_strObjName)
{
	bool bRetVal = true;

	int SuffixPos = a_strObjName.find("<");

	assert(SuffixPos != -1);

	if (SuffixPos != -1)
	{
		std::string strSuffix  = a_strObjName.substr(SuffixPos);

		if (strcmp(strSuffix.c_str(), V_POSX) == 0 ||
			strcmp(strSuffix.c_str(), V_POSY) == 0 ||
			strcmp(strSuffix.c_str(), V_POSZ) == 0)
		{
			// It's a marker position... anything else will indicate a body...

			bRetVal = false;
		}
	}

	return bRetVal;
}

bool VFileTranslator::IsGlobal(std::string a_strObjName)
{
	bool bRetVal = false;

	int SuffixPos = a_strObjName.find("<");

	assert(SuffixPos != -1);

	if (SuffixPos != -1)
	{
		std::string strSuffix  = a_strObjName.substr(SuffixPos);

		if (strcmp(strSuffix.c_str(), V_ROTX) == 0 ||
			strcmp(strSuffix.c_str(), V_ROTY) == 0 ||
			strcmp(strSuffix.c_str(), V_ROTZ) == 0 || 
			strcmp(strSuffix.c_str(), V_TRANSX) == 0 ||
			strcmp(strSuffix.c_str(), V_TRANSY) == 0 ||
			strcmp(strSuffix.c_str(), V_TRANSZ) == 0)
		{
			// It's global...

			bRetVal = true;
		}
	}

	return bRetVal;
}

std::string VFileTranslator::StripSuffix(std::string a_strDofLabel)
{	
	int SuffixPos = a_strDofLabel.rfind("<");

	assert(SuffixPos != -1); // err... this label doesn't seem to have a suffix....??.. 

	if (SuffixPos != -1)
	{
		// Okay.. we've found the suffix, now trim it
	
		a_strDofLabel = a_strDofLabel.substr(0, SuffixPos);

		// Oh, and trim off any trailing whitespace

		a_strDofLabel = TrimRight(a_strDofLabel);
	}

	// Just check that the name is legal
	assert(a_strDofLabel.find("<") == -1);

	return a_strDofLabel;
}

void VFileTranslator::SetupKeyframes(VFileContents * a_pContents, int a_CurDatagroup)
{
	VFileDynamicRecord * pRecord = NULL;
	int MinFrame = a_pContents->GetMinFrameNum();
	int MaxFrame = a_pContents->GetMaxFrameNum();
	int FrameRate = (int)a_pContents->GetFrameRate(a_CurDatagroup);

	for (int i = MinFrame; i < MaxFrame; i++)
	{
		pRecord = a_pContents->GetDynamicRecord(a_CurDatagroup, i);
		
// Ack... complications.... what about stuff that is NOT double??

		if (pRecord == NULL)
			continue;

		// Step through all the bodies and set key for this frame

		std::vector<VMayaBody *>::iterator iBody;

		for (iBody = m_Bodies.begin(); iBody != m_Bodies.end(); iBody++)
		{
			(*iBody)->SetKeyframe(*pRecord, FrameRate);
		}
	}
}

